@@ -18,16 +18,16 @@ test_that("XML SystemMetadata parsing works", {
   sysmeta <- new("SystemMetadata")
   expect_that(sysmeta@serialVersion, equals(1))
   doc <- xmlParseDoc("../testfiles/sysmeta.xml", asText=FALSE)
-  expect_that(xmlValue(xmlRoot(doc)[["identifier"]]), matches(testid))
+  expect_match(xmlValue(xmlRoot(doc)[["identifier"]]), testid)
   xml <- xmlRoot(doc)
   #getEncoding(doc)
   sysmeta <- parseSystemMetadata(sysmeta, xmlRoot(xml))
-  expect_that(sysmeta@identifier, matches(testid))
+  expect_match(sysmeta@identifier, testid)
   expect_that(nrow(sysmeta@accessPolicy), equals(5))
-  expect_that(as.character(sysmeta@accessPolicy$permission[[1]]), matches("read"))
+  expect_match(as.character(sysmeta@accessPolicy$permission[[1]]), "read")
   expect_that(sysmeta@archived, is_true())
   csattrs <- xmlAttrs(xml[["checksum"]])
-  expect_that(sysmeta@checksumAlgorithm, matches(csattrs[[1]]))
+  expect_match(sysmeta@checksumAlgorithm, csattrs[[1]])
   expect_that(grep("urn:node:KNB", sysmeta@preferredNodes) > 0, is_true())
   expect_that(grep("urn:node:mnUNM1", sysmeta@preferredNodes) > 0, is_true())
   expect_that(grep("urn:node:BADNODE", sysmeta@blockedNodes) > 0, is_true())
@@ -35,24 +35,44 @@ test_that("XML SystemMetadata parsing works", {
   rm(xml)
   rm(doc)
   rm(csattrs)
+  
   # Parse v2.0 system metadata
   testid <- "0007f892-0d8f-4451-94e9-94d02ba5dd0d_0"
   sysmeta <- new("SystemMetadata")
   expect_that(sysmeta@serialVersion, equals(1))
   doc <- xmlParseDoc("../testfiles/sysmeta-v2.xml", asText=FALSE)
-  expect_that(xmlValue(xmlRoot(doc)[["identifier"]]), matches(testid))
+  expect_match(xmlValue(xmlRoot(doc)[["identifier"]]), testid)
   xml <- xmlRoot(doc)
   sysmeta <- parseSystemMetadata(sysmeta, xmlRoot(xml))
-  expect_that(sysmeta@identifier, matches(testid))
+  expect_match(sysmeta@identifier, testid)
   expect_that(nrow(sysmeta@accessPolicy), equals(1))
-  expect_that(as.character(sysmeta@accessPolicy$permission[[1]]), matches("read"))
+  expect_match(as.character(sysmeta@accessPolicy$permission[[1]]), "read")
   expect_false(sysmeta@archived)
   csattrs <- xmlAttrs(xml[["checksum"]])
-  expect_that(sysmeta@checksumAlgorithm, matches(csattrs[[1]]))
+  expect_match(sysmeta@checksumAlgorithm, csattrs[[1]])
   expect_equal(sysmeta@seriesId, "3")
-  expect_equal(sysmeta@mediaType, "text/csv")
-  expect_equal(sysmeta@fileName, "testData.csv")
+  expect_equal(sysmeta@mediaType, "application/rdf+xml")
+  expect_equal(sysmeta@fileName, "testresmap.rdf")
   
+  # Parse v2.0 system metadata, checking parsing of replication policy
+  testid <- "0007f892-0d8f-4451-94e9-94d02ba5dd0d_0"
+  sysmeta <- new("SystemMetadata")
+  expect_that(sysmeta@serialVersion, equals(1))
+  doc <- xmlParseDoc("../testfiles/sysmeta-v2-repfalse.xml", asText=FALSE)
+  expect_match(xmlValue(xmlRoot(doc)[["identifier"]]), testid)
+  xml <- xmlRoot(doc)
+  sysmeta <- parseSystemMetadata(sysmeta, xmlRoot(xml))
+  expect_false(sysmeta@replicationAllowed)
+  
+  # Parse v2.0 system metadata, checking parsing when missing numReplicas
+  testid <- "arctic-data.9794.1"
+  sysmeta <- new("SystemMetadata")
+  expect_that(sysmeta@serialVersion, equals(1))
+  doc <- xmlParseDoc("../testfiles/sysmeta-v2-repfalse-zero-reps.xml", asText=FALSE)
+  expect_match(xmlValue(xmlRoot(doc)[["identifier"]]), testid)
+  xml <- xmlRoot(doc)
+  sysmeta <- parseSystemMetadata(sysmeta, xmlRoot(xml))
+  expect_false(sysmeta@replicationAllowed)
 })
 
 test_that("XML SystemMetadata serialization works", {
@@ -62,9 +82,9 @@ test_that("XML SystemMetadata serialization works", {
     sysmeta <- new("SystemMetadata")
     expect_that(sysmeta@serialVersion, equals(1))
     xml <- xmlParseDoc("../testfiles/sysmeta.xml", asText=FALSE)
-    expect_that(xmlValue(xmlRoot(xml)[["identifier"]]), matches(testid))
+    expect_match(xmlValue(xmlRoot(xml)[["identifier"]]), testid)
     sysmeta <- parseSystemMetadata(sysmeta, xmlRoot(xml))
-    expect_that(sysmeta@identifier, matches(testid))
+    expect_match(sysmeta@identifier, testid)
     expect_that(sysmeta@archived, is_true())
     # Check if the access policy is serialized grouped by subjects
     sysmeta <- addAccessRule(sysmeta, "bob", "read")
@@ -82,13 +102,13 @@ test_that("XML SystemMetadata serialization works", {
     expect_equal(xml, xmlRef)
     #cat(xml)
     # Search for specific, expected items in the serialized sysmeta
-    expect_that(xml, matches("<d1:systemMetadata"))
-    expect_that(xml, matches("<blockedMemberNode>urn:node:BADNODE</blockedMemberNode>"))
-    expect_that(xml, matches("<preferredMemberNode>urn:node:KNB</preferredMemberNode>"))
-    expect_that(xml, matches("<subject>public</subject>"))
-    expect_that(xml, matches("<permission>read</permission>"))
-    expect_that(xml, matches("<subject>CN=Subject2,O=Google,C=US,DC=cilogon,DC=org</subject>"))
-    expect_that(xml, matches("<permission>changePermission</permission>"))
+    expect_match(xml, "<d1:systemMetadata")
+    expect_match(xml, "<blockedMemberNode>urn:node:BADNODE</blockedMemberNode>")
+    expect_match(xml, "<preferredMemberNode>urn:node:KNB</preferredMemberNode>")
+    expect_match(xml, "<subject>public</subject>")
+    expect_match(xml, "<permission>read</permission>")
+    expect_match(xml, "<subject>CN=Subject2,O=Google,C=US,DC=cilogon,DC=org</subject>")
+    expect_match(xml, "<permission>changePermission</permission>")
     sysmeta@obsoletes <- ""
     sysmeta <- new("SystemMetadata")
     xml <- serializeSystemMetadata(sysmeta)
@@ -100,15 +120,15 @@ test_that("SystemMetadata XML constructor works", {
     library(datapack)
     testid <- "doi:10.xxyy/AA/tesdoc123456789"
     doc <- xmlParseDoc("../testfiles/sysmeta.xml", asText=FALSE)
-    expect_that(xmlValue(xmlRoot(doc)[["identifier"]]), matches(testid))
+    expect_match(xmlValue(xmlRoot(doc)[["identifier"]]), testid)
     xml <- xmlRoot(doc)
     sysmeta <- SystemMetadata(xmlRoot(xml))
-    expect_that(sysmeta@identifier, matches(testid))
+    expect_match(sysmeta@identifier, testid)
     expect_that(nrow(sysmeta@accessPolicy), equals(5))
-    expect_that(as.character(sysmeta@accessPolicy$permission[[1]]), matches("read"))
+    expect_match(as.character(sysmeta@accessPolicy$permission[[1]]), "read")
     expect_that(sysmeta@archived, is_true())
     csattrs <- xmlAttrs(xml[["checksum"]])
-    expect_that(sysmeta@checksumAlgorithm, matches(csattrs[[1]]))
+    expect_match(sysmeta@checksumAlgorithm, csattrs[[1]])
     expect_that(grep("urn:node:KNB", sysmeta@preferredNodes) > 0, is_true())
     expect_that(grep("urn:node:mnUNM1", sysmeta@preferredNodes) > 0, is_true())
     expect_that(grep("urn:node:BADNODE", sysmeta@blockedNodes) > 0, is_true())
@@ -131,14 +151,14 @@ test_that("SystemMetadata accessPolicy can be constructed and changed", {
     sysmeta <- new("SystemMetadata", identifier="foo", formatId="text/csv", size=59, checksum="jdhdjhfd", rightsHolder="ff", accessPolicy=apolicy)
     expect_that(sysmeta@serialVersion, equals(1))
     expect_that(nrow(sysmeta@accessPolicy), equals(1))
-    expect_that(as.character(sysmeta@accessPolicy$permission[[1]]), matches("read"))
+    expect_match(as.character(sysmeta@accessPolicy$permission[[1]]), "read")
     
     sysmeta <- addAccessRule(sysmeta, "foo", "write")
     expect_that(nrow(sysmeta@accessPolicy), equals(2))
     # Try to add same rule again and make sure it didn't get duplicated
     sysmeta <- addAccessRule(sysmeta, "foo", "write")
     expect_that(nrow(sysmeta@accessPolicy), equals(2))
-    expect_that(as.character(sysmeta@accessPolicy$permission[[2]]), matches("write"))
+    expect_match(as.character(sysmeta@accessPolicy$permission[[2]]), "write")
     expect_true(hasAccessRule(sysmeta, "foo", "write"))
     apolicy=data.frame(subject=c("bar", "baz"), permission= c("changePermission", "write"))
     sysmeta <- addAccessRule(sysmeta, apolicy)
@@ -148,7 +168,15 @@ test_that("SystemMetadata accessPolicy can be constructed and changed", {
     expect_true(hasAccessRule(sysmeta, "baz", "write"))
     expect_true(!hasAccessRule(sysmeta, "baz", "changePermission"))
     expect_that(nrow(sysmeta@accessPolicy), equals(4))
-    expect_that(as.character(sysmeta@accessPolicy$permission[[4]]), matches("write"))
-    expect_that(as.character(sysmeta@accessPolicy$subject[[4]]), matches("baz"))    
+    expect_match(as.character(sysmeta@accessPolicy$permission[[4]]), "write")
+    expect_match(as.character(sysmeta@accessPolicy$subject[[4]]), "baz")    
 })
 
+test_that("SystemMetadata accessPolicy can be cleared", {
+    sysmeta <- new("SystemMetadata")
+    sysmeta <- addAccessRule(sysmeta, "public", "read")
+    expect_true(nrow(sysmeta@accessPolicy) == 1)
+    
+    sysmeta <- clearAccessPolicy(sysmeta)
+    expect_true(nrow(sysmeta@accessPolicy) == 0)
+})
